import epikit
import yaml
import os
from datetime import datetime


class ensemble():

    __version__ = "1.2"

    def __init__(self, infile):

        self.input = infile

        # load data, if present
        with open(infile, 'r') as initdata:
            data = yaml.safe_load(initdata)

        if data:
            for key, value in data.items():
                setattr(self, key, value)

    def write(self, dest):
        with open(dest, 'w') as file:
            yaml.dump(self.__dict__, file)

    def create_ensemble_metadata(self, destination):
        os.makedirs(destination, exist_ok=True)
        self.write_ensemble_metadata(destination)
        self.write_experiments_metadata(destination)

        for e in self.ensemble["experiments"]:
            self.write_experiment_metadata(destination, e)

    def write_ensemble_metadata(self, destination):
        now = datetime.now()
        data = {
            "description": {
                "title": self.ensemble["meta"]["title"],
                "desc":  self.ensemble["meta"]["desc"]
            },
            "release": {
                "date": now.strftime("%Y-%m-%d"),
                "time": now.strftime("%H:%M:%S")
            },
            "hierarchy": {
                "version": epikit.ensemble.__version__
            }
        }
        with open(destination + "/" + "meta.yaml", 'w') as metadata:
            yaml.dump(data, metadata, sort_keys=False)

    def write_experiments_metadata(self, destination):
        print("writing experiments metadata")

    def write_experiment_metadata(self, destination, experiment):
        print("writing experiment metadata")
        print("  " + experiment["name"])




import epikit
import yaml
import os
from datetime import datetime

class inputdeck():

    def __init__(self, infile):

        self.input = infile

        # load data, if present
        with open(infile, 'r') as initdata:
            data = yaml.safe_load(initdata)

        if data:
            for key, value in data.items():
                setattr(self, key, value)

    def write(self, dest):
        with open(dest, 'w') as file:
            yaml.dump(self.__dict__, file)

    def create_ensemble_metadata(self, destination):
        os.makedirs(destination, exist_ok=True)
        self.write_ensemble_metadata(destination)
        self.write_experiments_metadata(destination)

        for e in self.ensemble["experiments"]:
            self.write_experiment_metadata(destination, e)

    def write_ensemble_metadata(self, destination):
        now = datetime.now()
        data = {
            "description": {
                "title": self.ensemble["meta"]["title"],
                "desc":  self.ensemble["meta"]["desc"]
            },
            "release": {
                "date": now.strftime("%Y-%m-%d"),
                "time": now.strftime("%H:%M:%S")
            },
            "hierarchy": {
                "version": "v1.0.0"
            }
        }
        with open(destination + "/" + "meta.yaml", 'w') as metadata:
            yaml.dump(data, metadata, sort_keys=False)

    def write_experiments_metadata(self, destination):
        print("writing experiments metadata")

    def write_experiment_metadata(self, destination, experiment):
        print("writing experiment metadata")
        print("  " + experiment["name"])



class workflow():

    def __init__(self, infile="workflow.init", cell_line=None, description=None, experiment=None, replicate=0, 
                 resolution=100000, rootdir=",", timeunits='hrs', timevalues=[], treatments=[], ):

        # set defaults 
        self.cell_line   = cell_line 
        self.description = description
        self.experiment  = experiment 
        self.replicate   = replicate 
        self.resolution  = resolution
        self.rootdir     = os.path.abspath(rootdir)
        self.timeunits   = timeunits
        self.timevalues  = timevalues
        self.treatments  = treatments 
        self.version     = epikit.__version__

        # load data, if present
        with open(self.rootdir + "/" + infile, 'r') as initdata:
            data = yaml.safe_load(initdata)

        if data:
            for key, value in data.items():
                setattr(self, key, value)

        self.add_experimental_design()

    def __str__(self):
        return f"cell_line: {self.cell_line}\ndescription: {self.description}\nexperiment: {self.experiment}\nreplicate: {self.replicate}\nresolution: {self.resolution}\nrootdir: {self.rootdir}\ntimeunits: {self.timeunits}\ntimevalues: {self.timevalues}\ntreatments: {self.treatments}\nversion: {self.version}\ndatasets: " + str(self.datasets)

    def add_experimental_design(self):
        self.datasets = [] 
        with open(self.rootdir + "/experimental_design.csv") as ed:
            # this reader will skip the first line of a csv file (the column names)
            edfile = csv.DictReader(ed, delimiter=',')
            self.datasets.append([])
            self.datasets.append([])
            for row in edfile:
                self.datasets[0].append(row['filename_0'])
                self.datasets[1].append(row['filename_1'])

    def write(self, dest):
        with open(dest, 'w') as file:
            yaml.dump(self.__dict__, file)



import os
import glob
import yaml
import argparse
import sys

DATA_VERSION_CHECKED = "1.2"

# Check that all files contain a set of strings 
def check_strings_in_files(strings, files):
    """
    Check if each string in `strings` is present in each file in `files`.

    Parameters:
        strings (list): A list of strings to search for.
        files (list): A list of file paths.

    Returns:
        dict: A nested dictionary mapping {filename: {string: True/False}}.
    """
    results = {}

    for file_path in files:
        if not os.path.isfile(file_path):
            raise ValueError(f"{file_path} is not a valid file.")

        results[file_path] = {}
        with open(file_path, "r", encoding="utf-8", errors="ignore") as f:
            content = f.read()

            for s in strings:
                results[file_path][s] = s in content

    return results

def check_directories_have_same_contents(directories):
    """
    Check if all given directories contain the same contents 

    Parameters:
        directories (list): A list of directory paths.

    Returns:
        bool: True if all directories have the same files, False otherwise.
        dict: A dictionary mapping each directory to its set of files.
    """
    dir_files = {}
    
    # Collect contents 
    for d in directories:
        if not os.path.isdir(d):
            raise ValueError(f"{d} is not a valid directory.")
        dir_files[d] = set(os.listdir(d))
        if False:
            print(os.listdir(d))
    
    # Compare sets of files
    all_file_sets = list(dir_files.values())
    first_set = all_file_sets[0]
    all_same = all(fs == first_set for fs in all_file_sets)

    return all_same, dir_files

def main():
    parser = argparse.ArgumentParser(
        description="Script that requires an ensemble argument."
    )
    
    # Add required argument
    parser.add_argument( "--ensemble", required=False, help="Name of the ensemble to use.")
    parser.add_argument( "--verbose", action="store_true",   help="Enable verbose output.")
    parser.add_argument( "-v", "--version", action="store_true", help="Print script version and exit")
    args = parser.parse_args()

    if args.version:
        print(f"data version checked: {DATA_VERSION_CHECKED}")
        sys.exit(0)
    
    if not args.ensemble:
        parse.error("the following argument is required: --ensemble")

    # Access the value
    print(f"Checking integrity of ensemble: {args.ensemble}")

    # check directories
    directories = glob.glob(os.path.join(args.ensemble, 'experiments/*/'), recursive=True)

    # Print the found directories
    if args.verbose: 
        print(directories)

    success = True

    # timesteps
    print("Checking experiment directories for timesteps")
    same, files_per_dir = check_directories_have_same_contents(directories)
    if same:
        print("  All experiments have the same timesteps.")
    else:
        print("  All experiments DO NOT have the same timesteps.")
        success = False
        for d, files in files_per_dir.items():
            print(f"{d}: {files}")

    # contents of timesteps
    print("Checking contents of timestep directories")
    timesteps = glob.glob(os.path.join(args.ensemble, 'experiments/*/*/'), recursive=True)
    same, files_per_dir = check_directories_have_same_contents(timesteps)
    if same:
        print("  All experiment/timestep directories contain the same files.")
    else:
        print("  All experiment/timestep directories DO NOT contain the same files.")
        success = False
        for d, files in files_per_dir.items():
            print(f"{d}: {files}")

    # check chromosome list against all relevant files
    print("Checking that all relevant files have required chromosomes")
    files_to_check = glob.glob(os.path.join(args.ensemble, 'experiments/*/*/*'), recursive=True)

    # Load YAML file
    metafile = os.path.join(args.ensemble, "experiments", "meta.yaml") 
    with open(metafile, "r") as f:
        data = yaml.safe_load(f)
    strings_to_check = data["structure"]["chromosomes"] 

    # perform the check
    results = check_strings_in_files(strings_to_check, files_to_check)

    # Print results nicely
    found = True
    for file, checks in results.items():
        # print(f"\nFile: {file}")
        for s, found in checks.items():
            if found:
                status = "found" 
            else:
                found = False
                status = "not found"

    if found:
        print("  All chromosomes found in all files")
    else:
        success = False
        print("  All chromosomes NOT found in all files")

    # final evaluation
    print("")
    print("Result")
    if success:
        print("  PASS")
        exit(0)
    else:
        print("  FAIL")
        exit(1)




import csv
import argparse
import glob
import os
import sys

DATA_VERSION_CHECKED = "1.2"

def unique_first_column(csv_file):
    """
    Find unique strings in the first column of a CSV file, ignoring the header.
    
    Parameters:
        csv_file (str): Path to the CSV file.
    
    Returns:
        set: A set of unique values from the first column.
    """
    unique_values = set()

    with open(csv_file, "r", newline="", encoding="utf-8") as f:
        reader = csv.reader(f)
        
        # Skip header
        next(reader, None)

        for row in reader:
            if row:  # avoid empty lines
                unique_values.add(row[0])  # first column

    return unique_values




def main():
    parser = argparse.ArgumentParser(
        description="tool that generates metadata for ensemble."
    )
    
    # Add required argument
    parser.add_argument( "--ensemble", required=False, help="Name of the ensemble to use.")
    parser.add_argument( "--verbose", action="store_true",   help="Enable verbose output.")
    parser.add_argument( "-v", "--version", action="store_true", help="Print script version and exit")
    args = parser.parse_args()

    if args.version:
        print(f"data version checked: {DATA_VERSION_CHECKED}")
        sys.exit(0)
    
    if not args.ensemble:
        parser.error("the following argument is required: --ensemble")

    # experiment meta.yaml
    expmeta = os.path.join(args.ensemble, "experiments/meta.yaml")
    print(f"Creating {expmeta} file")
    if not os.path.exists(expmeta):
        # find structure files
        structure_files = glob.glob(os.path.join(args.ensemble, 'experiments/*/*/structure.csv'), recursive=True)
        csv_path = structure_files[0] 
        unique_strings = unique_first_column(csv_path)
        sorted_unique_strings = sorted(unique_strings)

        with open(expmeta, 'w') as outfile:
            outfile.write("structure:\n")
            outfile.write("  chromosomes:\n")
            for s in sorted_unique_strings:
                outfile.write("    - " + s + "\n")
    else:
        print(f"  {expmeta} exists, not writing a new one ...")
